// Base64.h
/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _CGI_BASE64_H
#define _CGI_BASE64_H

#ifdef _WIN32
#define STL_USING_STRING
#define STL_USING_VECTOR
#include <stlhelper.h>
#else
#include <string>
#include <vector>
#endif


namespace cgi {

class Base64
{
public:
	Base64(char = '=');
	~Base64();

	// file to file
	void encode_file(const char *s,const char *outfn);

	// input buffer to std::vector<std::string>
	void encode64(const char *in,size_t len,std::vector<std::string> &out);

	// file to std::vector<std::string>
	void encode_from_file(const char *,std::vector<std::string> &);

	std::string encode_buffer(const std::string& );

	// buffer to std::string
//	void encode_buffer(const char *,size_t l,std::string& );

	// file to file
	void decode_file(char *s,char *outfn);

	// std::vector<std::string> to file
	void decode_to_file(std::vector<std::string> &body,char *outfn);

	// file to FILE *
	void decode_stdout(char *fn,FILE *out = stdout);

	// string to FILE *
	void decode_to_stdout(const std::string &,FILE * = stdout);

	// std::vector<std::string> to FILE *
	void decode_to_stdout(std::vector<std::string> &,FILE * = stdout);

	// string to buffer
	void decode_to_buffer(const std::string& ,char *,size_t max);

	size_t decode_length(const std::string& );

private:
	void setupbase64();
	char setupdecode(char c);
	unsigned char reverse(char c);
	void encode(const char *in,char *out);
	int decode(char c);
	void decode_string(char *s, unsigned char *output, int &l);
	char m_eqchar;
};

} // namespace

#endif // _CGI_BASE64_H
