/**
 **	Form.h - read stdin, parse cgi input
 **
 **	Written: 1999-Feb-10 grymse@alhem.net
 **/

/*
Copyright (C) 1999  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _FORM_H
#define _FORM_H

#include <stdio.h>
#ifdef _WIN32
#define STL_USING_STRING
#define STL_USING_LIST
#include <stlhelper.h>
#else
#include <string>
#include <list>
#endif


// socket input buffer size
#define IBUFSZ 10000


/**
 * Store the name/value pairs from a GET/POST operation.
 * "name" does not have to be unique.
*/

typedef struct cgistruct
{
//	struct cgistruct *next; //!< Linked list ptr to next object
	char *name; //!< Name of parameter
	char *value; //!< Value of parameter
	char *path; //!< Path to file if parameter was a posted file
} CGI;
typedef std::list<CGI *> cgi_v;


//! Process and store user input from a GET/POST operation.

class Form
{
public:
	/**
	 * Default constructor (used in POST operations).
	 * Input is read from stdin. Number of characters to read
	 * can be found in the environment variable CONTENT_LENGTH.
	*/
	Form(FILE * = stdin);
	/**
	 * Another constructor (used in GET operations).
	 * Input is read from the environment variable QUERY_STRING.
	 * @param query_string The httpd server provided QUERY_STRING
	 * @param length Query string length.
	*/
	Form(char *query_string,int length);

	~Form();

	void EnableRaw(bool);

	void strcpyval(char *,char *,size_t);

	/* get names */
	bool getfirst(char *,short);
	bool getnext(char *,short);

	/* get names and values */
	bool getfirst(char *,short,char *,short);
	bool getnext(char *,short,char *,short);

	/* get value */
	int getvalue(char *,char *,short);
	std::string getvalue(const std::string& );
	size_t getlength(char *);
//	CGI *getbase() { return cgibase; }
	cgi_v& getbase() { return m_cgi; }

	char *GetBoundary();

private:
	Form(const Form& ) {}
	Form& operator=(const Form& ) { return *this; }
//	CGI *cgibase;
	cgi_v m_cgi;
//	CGI *current;
	cgi_v::iterator m_current;
	char *m_strBoundary;
	bool raw;
};
#endif // _FORM_H
